]> git.cworth.org Git - empires-server/blobdiff - game.js
test: Maintain a list of activated players for automated cleanup
[empires-server] / game.js
diff --git a/game.js b/game.js
index 0946568e9bbd8adc5c99d82832261fa1ac254d5d..7ca150735269d0c0b6c4bc0681e70302d4a41813 100644 (file)
--- a/game.js
+++ b/game.js
@@ -9,6 +9,7 @@ class Player {
     this.name = name;
     this.connections = [connection];
     this.team = no_team;
+    this.active = true;
   }
 
   add_connection(connection) {
@@ -23,7 +24,7 @@ class Player {
 
   /* Returns the number of remaining connections after this one is removed. */
   remove_connection(connection) {
-    this.connections.filter(c => c !== connection);
+    this.connections = this.connections.filter(c => c !== connection);
     return this.connections.length;
   }
 
@@ -36,11 +37,22 @@ class Player {
     return JSON.stringify({
       id: this.id,
       name: this.name,
-      team: this.team.name
+      team: this.team.name,
+      score: this.score
     });
   }
 }
 
+/* This replacer function allows for JSON.stringify to give results
+ * for objects of various types that are used in game state.
+ */
+function stringify_replacer(key, value) {
+  if (typeof value === 'object' && value instanceof Set) {
+    return [...value];
+  }
+  return value;
+}
+
 /* Base class providing common code for game engine implementations. */
 class Game {
   constructor(id) {
@@ -158,7 +170,20 @@ class Game {
 
     /* No existing player. Add a new one. */
     const id = this.next_player_id;
-    const player = new Player(id, session.id, session.nickname, connection);
+    let nickname = session.nickname;
+    if (nickname === "")
+      nickname = "Guest";
+    const nickname_orig = nickname;
+
+    /* Ensure we don't have a name collision with a previous player. */
+    let unique_suffix = 1;
+    while (this.players.find(player => player.name === nickname))
+    {
+      nickname = `${nickname_orig}${unique_suffix.toString().padStart(2, '0')}`;
+      unique_suffix++;
+    }
+
+    const player = new Player(id, session.id, nickname, connection);
 
     /* Broadcast before adding player to list (to avoid announcing the
      * new player to itself). */
@@ -171,7 +196,7 @@ class Game {
 
     /* After adding the player to the list, and if we are already past
      * the first move, assign this player to the first team that
-     * doesn't already have a player aissgned (if any). */
+     * doesn't already have a player assigned (if any). */
     if (! this.first_move) {
       const have_players = Array(this.teams.length).fill(false);
       this.players.forEach(p => {
@@ -190,9 +215,8 @@ class Game {
   remove_player_connection(player, connection) {
     const remaining = player.remove_connection(connection);
     if (remaining === 0) {
+      player.active = false;
       const player_data = JSON.stringify({ id: player.id });
-      this.players.filter(p => p !== player);
-      delete this.players_by_session[player.session_id];
       this.broadcast_event("player-exit", player_data);
     }
   }
@@ -211,6 +235,10 @@ class Game {
     this.broadcast_string(`event: ${type}\ndata: ${data}\n`);
   }
 
+  broadcast_event_object(type, obj) {
+    this.broadcast_event(type, JSON.stringify(obj));
+  }
+
   handle_events(request, response) {
     /* These headers will keep the connection open so we can stream events. */
     const headers = {
@@ -238,15 +266,15 @@ class Game {
     /* And the player-info event. */
     response.write(`event: player-info\ndata: ${player.info_json()}\n\n`);
 
-    /* As well as player-enter events for all existing players. */
-    this.players.filter(p => p !== player).forEach(p => {
+    /* As well as player-enter events for all existing, active players. */
+    this.players.filter(p => p !== player && p.active).forEach(p => {
       response.write(`event: player-enter\ndata: ${p.info_json()}\n\n`);
     });
 
     /* Finally, if this game class has a "state" property, stream that
      * current state to the client. */
     if (this.state) {
-      const state_json = JSON.stringify(this.state);
+      const state_json = JSON.stringify(this.state, stringify_replacer);
       response.write(`event: game-state\ndata: ${state_json}\n\n`);
     }
   }
@@ -293,7 +321,7 @@ class Game {
   }
 
   broadcast_move(move) {
-    this.broadcast_event("move", move);
+    this.broadcast_event("move", JSON.stringify(move));
   }
 
 }